/*  anatomywizard_v022.js: Batch processing script for anatomical files for BrainVoyager 21 and higher
 
 Functionality: Create anatomical projects (*.vmr), resample to 1x1x1mm and transform to sagittal orientation
 also perform intensity inhomogeneity correction and /or transform to AC-PC and Talairach space or MNI space.
 Next version: cortex-based alignment (CBA): aligning curvatures (*.smp) of surface files (*.srf). (Still needs testing, is switched off.)
 
 Installation: This batch processing bundle consists of anatomywizard.js and anatomywizard.ui, place in
 /Documents/BrainVoyager/Extensions/Scripts/ folder.
 
 Usage: In the BrainVoyager menu, select 'Scripts' > 'Edit and run scripts...' then 'anatomywizard'; then select the required processes via checkboxes on the first tab, configure the parameters (if necessary) for VMR creation and resampling ("isovoxelation") and load the required files (first set the number of projects). Please note that once the files have been loaded, a text file is saved in the directory of the first file, so that later the filenames can just be selected by loading the text file with filenames instead of selecting each file separately.
 
 Notes:
 a. Sometimes BrainVoyager does not accept any other files than *.dcm in a directory when creating projects.
 b. If any file that should be created by the anatomywizard with the same filename already exist, they will be overwritten without asking.
 c. When multiple *.vmr projects are created, they should have the same acquisition parameters for using this wizard. Same parameters for the projects are not necessary if the 'Create projects' step is skipped.
 
 Latest update:
 05-06-23: support for NIfTI; added tooltip to 'start' checkbox; will skip project creation step if *.nii(.gz) is provided in text file; removed redundant code
 12-05-23: only write new filenames to array and text file when processing succeeded
 11-05-23: line 583: change CreateProjectVMR() to CreateDocumentVMR(); line 127: replace 'filetype' variable by "*.txt" string
 15-04-19: added function and checkbox for transformation to MNI space; works fine when only doing isovoxelation, but not after also selecting ToSAG. >>>
 21-03-19: adapt code for BrainVoyager 21.
 19-10-2014: adaptation to BrowseFile() -> now script only; and moved to ISO and to SAG within Create VMR because of memory issues.
 22-08-2013: v0.2.1: in fact no changes, but this is compilation for Mac OS 10.8
 22-07-2013: v0.2 finished
 19-07-2013: v0.1 finished
 20-06-2013: creation date
 
 HB, Brain Innovation, 2013-2023.
 */

var scriptObj = new Object; 
scriptObj.scriptNameForUser     = "anatomywizard"; 
scriptObj.dialogFileName        = "anatomywizard_v022.ui";
scriptObj.dialogAccessName      = "batchDlg"; 
BrainVoyager.PrintToLog("anatomywizard started...");

var projfilenames;// = new Array();
var projfilenamessize = 0;
var targetfilenames = new Array();
var targetfilenamessize = 0;
var cancel = false;
var extlen = 4; // extension length; usually 4 (".vmr", ".nii") but sometimes 7 (".nii.gz")
const UNKNOWN_SPACE             = 0;
const VOLUMETRIC_SPACE          = 1;
const SURFACE_SPACE             = 2;
const TEXTFILE_FOR_PROJECTCREATION      = 1;
const TEXTFILE_FOR_INHCORRECTION        = 2;
const TEXTFILE_FOR_RESAMPLING           = 3;
const TEXTFILE_FOR_REORIENTATION        = 4;
const TEXTFILE_FOR_NORMALISATION_VOLUMETRIC        = 5;//aACPC and aTAL
const TEXTFILE_FOR_SPHERE_FROM_RECOSM   = 6;
const TEXTFILE_FOR_CALC_CURVATURE       = 7;
const TEXTFILE_FOR_NORMALISATION_SPHERICAL = 8;//CBA


// global widgets
var startButton, cancelButton;
// widgets on global tab
var chbCreateProjects, chbIsovoxel, chbTrfToSag, chbIIHC, chbACPCTALtrf, chbMNItrf,
chbDoCBA, cmbCBAstep, rbtnIncludeRigidCBA, rbtnNotIncludeRigidCBA;
// widgets on create VMR projects tab
var cmbFileType, spbNrSlices, spbXRes, spbYRes, spbBytesPerPixel, chbBigEndian;
// widgets on volume transformations tab
var cmbIsovoxInterpol;
// widgets on surface tab
var spbSphereRes, chbGroupCurv, chbGroupMesh;

// widgets on files tab
var nroffilesBox, getFileButton, selectFilesNowBox, selectTextFileBox;
var selectFilesNowBox3, selectTextFileBox3, btnGetSRFSMPNames, btnMTCGetVMRName, lblVMRforSRFName, spbNrMTCs; 


// ~~~~~~~~~~~~~~~~~~~~~~~ File I/O ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.getNativeProcessingFiles = function() {
    
    BrainVoyager.PrintToLog("Get files...");
    var filetype = "*.*";
    if (chbCreateProjects.checked == 1) {
        var rawfiletype = cmbFileType.currentText;
        if (rawfiletype == "DICOM") {
            filetype = "*.dcm";
        } else if (rawfiletype == "SIEMENS") {
            filetype = "*.IMA";
        } else if (rawfiletype == "PHILIPS_REC") {
            filetype = "*.REC";
        } else if (rawfiletype == "GE_I") {
            filetype = "*.*";
        } else if (rawfiletype == "GE_MR") {
            filetype = "*.MR";
        } else if (rawfiletype == "ANALYZE") {
            filetype = "*.hdr";
        }
    } else {
        filetype = "*.vmr *.nii *.nii.gz"; // to test further
        //  filetype = "*.vmr";
    }
    
    // select files manually
    projfilenames = new Array();
    if (selectFilesNowBox.checked == 1) {
        
        var nroffiles = nroffilesBox.value;
        if (nroffiles < 1) BrainVoyager.PrintToLog("Please note: number of selected files is 0.");
        BrainVoyager.PrintToLog("Start selecting: " + nroffiles + " files...");
        var filecounter;
        // loop over indicated number of files
        for (filecounter = 0; filecounter < nroffiles; filecounter++) {
            // get raw data filename or *.vmr filename
            
            var projfilename = this.getFileName(filetype);
            if (projfilename.length > 0) projfilenames.push(projfilename);
            
        }
        this.writeToTextFile(projfilenames, TEXTFILE_FOR_PROJECTCREATION, "input");
        BrainVoyager.PrintToLog("Files selected. If all parameters have been set, the Start button can be clicked.");
        
        // read filenames from textfile
    } else if (selectTextFileBox.checked == 1) {
        
        var filename = BrainVoyager.BrowseFile("Please select the file", "*.txt");//filetype);
        BrainVoyager.PrintToLog("Selected: " + filename);
        this.readTextFileIntoProjectNamesArray(filename, TEXTFILE_FOR_PROJECTCREATION);
        if (filename.length == 0) {
            success = false;
        }
    }
}

scriptObj.writeToTextFile = function(projfilenames, kindoffiles, stage, targetfilenames) { 
    
    var filecounter;
    var success = true;
    
    // write the textfile to the path of the first file
    var filetype, name;
    filetype = projfilenames[0].substring((projfilenames[0].length-3), projfilenames[0].length);
    name = String(this.getSuperPath(projfilenames[0]) + "filenames_" + stage + "_" + filetype.toUpperCase() + ".txt");
    // name = String(this.getPath(projfilenames[0]) + "filenames_" + stage + "_" + filetype.toUpperCase() + ".txt");
    BrainVoyager.PrintToLog("Start writing file: " + name);
    var filenamesfile = new QFile(name);
    filenamesfile.open(new QIODevice.OpenMode(QIODevice.WriteOnly));
    var textstr = new QTextStream(filenamesfile);
    textstr.writeDouble(projfilenames.length);
    textstr.writeString("\n");
    if ((kindoffiles == TEXTFILE_FOR_PROJECTCREATION) || (kindoffiles == TEXTFILE_FOR_INHCORRECTION) ||
        (kindoffiles == TEXTFILE_FOR_RESAMPLING) || (kindoffiles == TEXTFILE_FOR_REORIENTATION) ||
        (kindoffiles == TEXTFILE_FOR_NORMALISATION_VOLUMETRIC)) {
        BrainVoyager.PrintToLog("Write processed anatomy names...");
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            textstr.writeString(projfilenames[filecounter] + "\n");
        }
    } else {
        BrainVoyager.PrintToLog("Kind of files to write to textfile unknown.");
        success = false;
    }
    filenamesfile.close();
    BrainVoyager.PrintToLog("Wrote " + name + ".");
    return success;
}

scriptObj.readTextFileIntoProjectNamesArray = function(textfilename, kindoffiles) {
    
    var filecounter;
    var success = true;
    BrainVoyager.PrintToLog("Start reading: " + textfilename);
    var name = String(textfilename); // circumvention of problem with implicit strings
    var filenamesfile = new QFile(name);
    filenamesfile.open(new QIODevice.OpenMode(QIODevice.ReadOnly));
    var textstr = new QTextStream(filenamesfile);
    var nroffiles = parseInt(textstr.readLine());
    BrainVoyager.PrintToLog("Number of projects: " + nroffiles);
    BrainVoyager.PrintToLog("Kind of projects: " + ((kindoffiles == TEXTFILE_FOR_PROJECTCREATION)         ?   "project creation/preprocessing" :
                                                    (kindoffiles == TEXTFILE_FOR_INHCORRECTION)             ?   "inhomogeneity corrected" :
                                                    (kindoffiles == TEXTFILE_FOR_RESAMPLING)                ?   "isovoxel" :
                                                    (kindoffiles == TEXTFILE_FOR_REORIENTATION)             ?   "sagoriented" :
                                                    (kindoffiles == TEXTFILE_FOR_NORMALISATION_VOLUMETRIC)  ?   "volumetric normalisation" :
                                                    //   (kindoffiles == TEXTFILE_FOR_SURFACEPROCESSING) ?   "surface related" :
                                                    "unknown"));
    
    if ((kindoffiles == TEXTFILE_FOR_PROJECTCREATION) || (kindoffiles == TEXTFILE_FOR_INHCORRECTION) || (kindoffiles == TEXTFILE_FOR_RESAMPLING) ||
        (kindoffiles == TEXTFILE_FOR_REORIENTATION) || (kindoffiles == TEXTFILE_FOR_NORMALISATION_VOLUMETRIC)) {
        projfilenames = new Array();
        for (filecounter = 0; filecounter < nroffiles; filecounter++) {
            var filename = textstr.readLine();
            BrainVoyager.PrintToLog("File " + (filecounter+1) + ":\n" + filename);
            projfilenamessize = projfilenames.push(filename);
        } // end for
    } else {
        BrainVoyager.PrintToLog("Kind of files unknown.");
    }
    filenamesfile.close();
    return success;
}

scriptObj.getPath = function(filename) { 
    
    var start = 0; // start searching from begin
    var last = filename.lastIndexOf("/");
    var path = filename.substring(start, (last+1)); // now return path without filename
    return path;
}

scriptObj.getSuperPath = function(filename) { 
    // getSuperPath(): return 'higher' path including file separator "/"
    
    var start = 0; // start searching from begin
    var last = filename.lastIndexOf("/");
    var path = filename.substring(start, last); // now return path without filename
    last = path.lastIndexOf("/");
    var path2 = path.substring(start, last+1);
    return path2;
}

scriptObj.getVMRName = function() {
    
    var vmrname = this.getFileName("*.vmr");
    lblVMRname.text = vmrname;
}

scriptObj.getFileName = function(filetype) {
    
    var filename = BrainVoyager.BrowseFile("Please select the file", filetype);
    BrainVoyager.PrintToLog("Selected: " + filename);
    return filename;
}


// ~~~~~~~~~~~~~~~~~~~~~~~ help functions ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

scriptObj.noErrors = function() {
    
    var noerror;
    var errornr = BrainVoyager.GetPluginIntParameter("ErrorNr");
    if (errornr != -1) {
        noerror = true;
    } else {
        var errormsg = BrainVoyager.GetPluginStringParameter("ErrorMsg");
        BrainVoyager.PrintToLog(errormsg);
        noerror = false;
    }
    return noerror;
}     

scriptObj.getIsovoxInterpolationDegreeStr = function(index) {
    
    switch (index) {
        case 1: return "trilinear"; break;
        case 2: return "cubic spline"; break;
        case 3: return "sinc"; break;
        default: return "not known";
    }
}    


function closeAllProjects() {
    
    var pn = BrainVoyager.PrintToLog;
    BrainVoyager.PrintToLog("Closing all projects...");
    var docs = BrainVoyager.Documents;
    var n_docs = docs.Count;
    
    pn("Nr of available documents: " + n_docs);
    
    if(n_docs == 0)
        return;
    
    for(var i=0; i<n_docs; i++){
        var docs = BrainVoyager.Documents;
        var n_docs = docs.Count;
        var doc = docs.Item(i);
        doc.Close();
        i--;
    }
}


// ~~~~~~~~~~~~~~~~~~~~~~~ main processing function ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.startProcessing = function() {
    
    var success = true;
    BrainVoyager.PrintToLog("Start processing...");
    if (projfilenames.length == 0) {
        BrainVoyager.PrintToLog("No input files have been selected. Process cancelled.");
    } else {
        
        if (cmbSelectProcSpace.currentIndex == VOLUMETRIC_SPACE) {
            
            if (chbCreateProjects.checked == 1) {
                this.createProjects();
            }
            
            /* next block temporarily switched off, the IICH is performed in createProjects() because v16 disappears out of memory after closing VMR
            if (chbIIHC.checked == 1) {
                this.correctIntensityValuesOfAnatomy();
            }*/
            
            if (chbIsovoxel.checked == 1) {
                this.resampleVoxelsToIsotropicResolution();
            }
            
            if (chbTrfToSag.checked == 1) {
                success = this.transformImageToSagittalOrientation();
            }
            
            if (chbACPCTALtrf.checked == 1) {
                this.transformImageToACPCandTalairachSpace();
            }
            
            if (chbMNItrf.checked == 1) {
                this.transformImageToMNISpace();
            }
            
        } else {
            BrainVoyager.PrintToLog("No processing space selected. Process cancelled.");
        }
    }
    // closeAllProjects();
    this.batchDlg.close();// >>> this doesn't happen
} 


// ~~~~~~~~~~~~~~~~~~~~~~~ Anatomical preprocessing and transformations ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


scriptObj.correctIntensityValuesOfAnatomy = function() {
    
    
    if (!cancel) {
        
        BrainVoyager.PrintToLog("Start intensity inhomogeneity correction...");
        
        // run IIHC
        var project;
        var filecounter, bvqxproject, newname;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            
            BrainVoyager.PrintToLog("Start correcting " + (filecounter+1) + " from:\n" + projfilenames[filecounter]);
            this.firstfilename = projfilenames[filecounter];
            this.savingdir = this.getPath(this.firstfilename);
            //   newname = this.firstfilename.substring(0, (this.firstfilename.length-3)) + "vmr";
            newname = this.firstfilename.substring(0, (this.firstfilename.length-extlen)) + "_IIHC.vmr"; //! test
            BrainVoyager.PrintToLog("New name: " + newname);
            project = BrainVoyager.OpenDocument(this.firstfilename); // add path?
            
            try {
                var iihc_performed =  project.CorrectIntensityInhomogeneities();
                if (iihc_performed) {
                    projfilenames[filecounter] = newname;
                    extlen = 4;
                }
            } catch (error) {
                BrainVoyager.PrintToLog("Error occurred during (B1) inhomogeneity correction: " + error.message);
            }
            project.Close();
        } // end filecounter
        
        BrainVoyager.PrintToLog("Finished inhomogeneity correction.");
        this.writeToTextFile(projfilenames, TEXTFILE_FOR_INHCORRECTION, "inhomogeneitycorrection");
    } else {
        BrainVoyager.PrintToLog("Procedure was cancelled.");
    }
    
}


scriptObj.resampleVoxelsToIsotropicResolution = function() {
    
    
    if (!cancel) {
        
        // get parameter
        var interpolationdegree = cmbIsovoxInterpol.currentIndex; // check if it starts counting at 0 or 1! => BV starts at 1.
        
        // run resampling
        var project;
        var filecounter, bvqxproject, newname;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            
            BrainVoyager.PrintToLog("Start resampling " + (filecounter+1) + " from:\n" + projfilenames[filecounter] + "\nwith interpolation degree: " +
                                    this.getIsovoxInterpolationDegreeStr(interpolationdegree));
            this.firstfilename = projfilenames[filecounter];
            this.savingdir = this.getPath(this.firstfilename);
            newname = this.firstfilename.substring(0, (this.firstfilename.length-extlen)) + "_ISO.vmr";
            BrainVoyager.PrintToLog("New name: " + newname);
            project = BrainVoyager.OpenDocument(this.firstfilename); // add path?
            success = project.AutoTransformToIsoVoxel(interpolationdegree, newname);//Interpolation for isovoxelation: 1 - trilinear, 2 - cubic spline (recommended), 3 - sinc
            if (success) {
                projfilenames[filecounter] = newname; // if it does not succeed, leave old filename in array; but this could cause problems later on
                extlen = 4;
            } else {
                BrainVoyager.PrintToLog("Resampling to isotropic voxels did not succeed.");
            }
            project.Close();
            
        } // end filecounter
        
        BrainVoyager.PrintToLog("Finished resampling.");
        this.writeToTextFile(projfilenames, TEXTFILE_FOR_RESAMPLING, "isovoxel");
        
    } else {
        BrainVoyager.PrintToLog("Procedure was cancelled.");
    }
    
}  


scriptObj.transformImageToSagittalOrientation = function() {
    
    var success = true;
    if (!cancel) {
        
        // apply transformation
        var project;
        var filecounter, bvqxproject, newname;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            
            BrainVoyager.PrintToLog("Start reorienting " + (filecounter+1) + " from:\n" + projfilenames[filecounter]);
            this.firstfilename = projfilenames[filecounter];
            this.savingdir = this.getPath(this.firstfilename);
            newname = this.firstfilename.substring(0, (this.firstfilename.length-4)) + "_SAG.vmr";
            BrainVoyager.PrintToLog("New name: " + newname);
            var ffilename = new QFile(this.firstfilename);
            if (ffilename.exists()) {
                project = BrainVoyager.OpenDocument(this.firstfilename);
                success = project.AutoTransformToSAG(newname);
                if (!success) {
                    BrainVoyager.PrintToLog("Transformation to sagittal orientation did not succeed.");
                    //          newname = projfilenames[filecounter]; // set newname to old name
                } else {
                    projfilenames[filecounter] = newname;// if it does not succeed, leave old filename in array; but this could cause problems later on
                    extlen = 4;
                }
                project.Close();
            }
        } // end filecounter
        
        if (success) {
            BrainVoyager.PrintToLog("Finished rotation to sagittal orientation.");
            this.writeToTextFile(projfilenames, TEXTFILE_FOR_REORIENTATION, "sagoriented");
        }
    } else {
        BrainVoyager.PrintToLog("Procedure was cancelled.");
    }
    return success;
} 

scriptObj.transformImageToACPCandTalairachSpace = function() {
    
    if (!cancel) {
        
        BrainVoyager.PrintToLog("Start transformation of functional data to AC-PC and Talairach space...");
        
        // run aACPC and aTAL
        var project;
        var filecounter, bvqxproject, newname;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            
            BrainVoyager.PrintToLog("Start correcting " + (filecounter+1) + " from:\n" + projfilenames[filecounter]);
            this.firstfilename = projfilenames[filecounter];
            this.savingdir = this.getPath(this.firstfilename);
            //   newname = this.firstfilename.substring(0, (this.firstfilename.length-3)) + "vmr";
            newname = this.firstfilename.substring(0, (this.firstfilename.length-extlen)) + "_aTAL.vmr"; //! test
            BrainVoyager.PrintToLog("New name: " + newname);
            project = BrainVoyager.OpenDocument(this.firstfilename); // add path?
            success = project.AutoACPCAndTALTransformation();
            if (success) {
                projfilenames[filecounter] = newname;
                BrainVoyager.PrintToLog("Finished volumetric normalisation.");
                extlen = 4;
                this.writeToTextFile(projfilenames, TEXTFILE_FOR_NORMALISATION_VOLUMETRIC, "talairach");
            } else {
                BrainVoyager.PrintToLog("AC-PC and Talairach transformation did not succeed.");
            }
            project.Close();
            // closeAllProjects();
        } // end filecounter
        
    } else {
        BrainVoyager.PrintToLog("Procedure was cancelled.");
    }
    
}

scriptObj.transformImageToMNISpace = function() {
    
    if (!cancel) {
        
        BrainVoyager.PrintToLog("Start transformation of functional data to MNI space...");
        var project;
        var filecounter, bvqxproject, newname;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            BrainVoyager.PrintToLog("Start correcting " + (filecounter+1) + " from:\n" + projfilenames[filecounter]);
            this.firstfilename = projfilenames[filecounter];
            this.savingdir = this.getPath(this.firstfilename);
            newname = this.firstfilename.substring(0, (this.firstfilename.length-4)) + "_MNI.vmr"; //! test
            BrainVoyager.PrintToLog("New name: " + newname);
            project = BrainVoyager.OpenDocument(this.firstfilename); // add path?
            success = project.NormalizeToMNISpace();
            if (success) {
                projfilenames[filecounter] = newname;
                BrainVoyager.PrintToLog("Finished volumetric normalisation.");
                extlen = 4;
                this.writeToTextFile(projfilenames, TEXTFILE_FOR_NORMALISATION_VOLUMETRIC, "mni");
            }
            project.Close();
            //  closeAllProjects();
        } // end filecounter
    } else {
        BrainVoyager.PrintToLog("Procedure was cancelled.");
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~ Create VMRs ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.createProjects = function() {
    
    // properties of ProjectFactory class
    var currentprojecttype;
    
    // declare and initialize member properties
    var filetype;
    var firstfilename;
    var nrslices;
    var stcprefix;
    var swapbytes;
    var nrbytes;
    var savingdir;
    var imgresx;
    var imgresy;
    var dostep = true;
    
    BrainVoyager.PrintToLog("Retrieve create project parameters...");
    
    // project-specific parameters
    this.filetype   = cmbFileType.currentText;
    this.nrslices   = spbNrSlices.value;
    this.swapbytes  = chbBigEndian.checked;
    this.nrbytes    = spbBytesPerPixel.value;
    this.imgresx    = spbXRes.value;
    this.imgresy    = spbYRes.value;
    
    // create VMR
    var project;
    var filecounter, bvqxproject, newname;
    for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
        
        BrainVoyager.PrintToLog("Start creating project " + (filecounter+1) + " from:\n" + projfilenames[filecounter]);
        this.firstfilename = projfilenames[filecounter];
        
        var part = this.firstfilename.slice(this.firstfilename.length-7, this.firstfilename.length);
        if (part === ".nii.gz") {
            dostep = false;
            extlen = 7;
        }
        part = this.firstfilename.slice(this.firstfilename.length-4, this.firstfilename.length);
        if (part === ".nii") {
            dostep = false;
        }
        
        if (dostep) {
            this.savingdir = this.getPath(this.firstfilename);
            newname = this.firstfilename.substring(0, (this.firstfilename.length-3)) + "vmr";
            BrainVoyager.PrintToLog("New name: " + newname);
            
            var projparams = "Project parameters:\n";
            projparams += "\tFiletype: " + this.filetype + "\n";
            projparams += "\tFirst file name: " + this.firstfilename + "\n";
            projparams += "\tNumber of slices: " + this.nrslices + "\n";
            projparams += "\tSwap bytes: " + this.swapbytes + "\n";
            projparams += "\tNumber of bytes: " + this.nrbytes + "\n";
            projparams += "\tSaving directory: " + this.savingdir + "\n";
            projparams += "\tSize of individual image (x-axis): " + this.imgresx + "\n";
            projparams += "\tSize of individual image (y-axis): " + this.imgresy + "\n";
            BrainVoyager.PrintToLog(projparams);
            
            project = BrainVoyager.CreateDocumentVMR(this.filetype, this.firstfilename, this.nrslices, this.swapbytes, this.imgresx, this.imgresy, this.nrbytes);
            project.SaveAs(newname);
            projfilenames[filecounter] = newname;
            
            // fix for bug that IIHC cannot take place when *.v16 is not loaded
            if (chbIIHC.checked == 1) {
             
                newname = this.firstfilename.substring(0, (this.firstfilename.length-4)) + "_IIHC.vmr";
                BrainVoyager.PrintToLog("New name: " + newname);
                success = project.CorrectIntensityInhomogeneities();
                BrainVoyager.PrintToLog("Correct B1 success: " + success);
                if (success) {
                    projfilenames[filecounter] = newname;
                    extlen = 4;
                }
            }
            project.Close();
        }
        
    } // end filecounter
    this.writeToTextFile(projfilenames, TEXTFILE_FOR_PROJECTCREATION, "projectcreation");
} 



// ~~~~~~~~~~~~~~~~~~~~~~~ main ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.initDlg = function() { 
    
    this.batchDlg.windowTitle = "anatomywizard 0.2.2";
    
    // ~ global widgets
    cmbSelectProcSpace          = BrainVoyager.FindChild("cmbSelectProcSpace");
    
    // native space
    chbCreateProjects           = BrainVoyager.FindChild("chbCreateProjects");
    chbIsovoxel                 = BrainVoyager.FindChild("chbIsovoxel");
    chbTrfToSag                 = BrainVoyager.FindChild("chbTrfToSag");
    chbIIHC                     = BrainVoyager.FindChild("chbIIHC");
    chbACPCTALtrf               = BrainVoyager.FindChild("chbACPCTALtrf");
    chbMNItrf 			        = BrainVoyager.FindChild("chbMNItrf");
    
    // surface space
    chbDoCBA                = BrainVoyager.FindChild("chbDoCBA");
    cmbCBAstep              = BrainVoyager.FindChild("cmbCBAstep");
    rbtnIncludeRigidCBA     = BrainVoyager.FindChild("rbtnIncludeRigidCBA");
    rbtnNotIncludeRigidCBA  = BrainVoyager.FindChild("rbtnNotIncludeRigidCBA");
    
    // ~ widgets on create projects tab
    cmbFileType             = BrainVoyager.FindChild("cmbFileType");
    chbBigEndian            = BrainVoyager.FindChild("chbBigEndian");
    spbNrSlices             = BrainVoyager.FindChild("spbNrSlices");
    spbXRes                 = BrainVoyager.FindChild("spbXRes");
    spbYRes                 = BrainVoyager.FindChild("spbYRes");
    spbBytesPerPixel        = BrainVoyager.FindChild("spbBytesPerPixel");
    
    // ~ widgets on volume transformations tab
    cmbIsovoxInterpol       = BrainVoyager.FindChild("cmbIsovoxInterpol");
    
    // ~ widgets on CBA tab
    spbSphereRes            = BrainVoyager.FindChild("dspbMTCFrom");
    chbGroupCurv            = BrainVoyager.FindChild("chbGroupCurv");
    chbGroupMesh            = BrainVoyager.FindChild("chbGroupMesh");
    
    // ~ widgets on files tab
    // volumetric space
    getFileButton       = BrainVoyager.FindChild("btnGetFiles");
    nroffilesBox        = BrainVoyager.FindChild("spnBoxNrOfFiles");
    selectFilesNowBox   = BrainVoyager.FindChild("rbtnFileDialog");
    selectTextFileBox   = BrainVoyager.FindChild("rbtnTextFile");
    // surface space
    selectFilesNowBox3      = BrainVoyager.FindChild("rbtnFileDialog_4");
    selectTextFileBox3      = BrainVoyager.FindChild("rbtnTextFile_4");
    btnGetSRFSMPNames       = BrainVoyager.FindChild("btnGetSRFSMPNames");
    lblVMRforSRFName        = BrainVoyager.FindChild("lblVMRforSRFName");
    btnVMRforSRFName        = BrainVoyager.FindChild("btnVMRforSRFName");
    spbNrMTCs               = BrainVoyager.FindChild("spbNrMTCs");
    
    // connect widgets to functions
    this.batchDlg.btnStart.clicked.connect(this, this.startProcessing); // might need to go to top of function
    getFileButton.clicked.connect(this, this.getNativeProcessingFiles);
    //  btnGetMTCnames.clicked.connect(this, this.getSurfaceProcessingFiles);
    btnVMRforSRFName.clicked.connect(this, this.getVMRName);
    
}

returnScriptObj = function() { 
    return scriptObj;
} 

returnScriptObj(); // should be after returnScriptOjb = function()
